package com.delfines.business;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import com.delfines.Constante;
import com.delfines.dao.ConnectionFactory;
import com.delfines.vo.Cliente;
import com.delfines.vo.DbfGeneracion;
import com.delfines.vo.Servicio;
import com.delfines.vo.Usuario;
import com.delfines.vo.Value;

public class ClienteBo {

	public Cliente buscarClienteSimple(Integer idCliente){
		Cliente obj = null; 
		
		PreparedStatement ps=null;
		ResultSet rs=null;

        try {
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("SELECT c.id_cliente,c.id_tipo_cliente,c.id_tipo_documento,c.nro_documento,c.denominacion,c.telefono,c.correo,c.direccion, ");
            query.append("tc.denominacion tipo_cliente,td.denominacion tipo_documento ");
            query.append("FROM dbo.cliente c ");
            query.append("INNER JOIN dbo.tipo_cliente tc ON (tc.id_tipo_cliente=c.id_tipo_cliente) ");
            query.append("INNER JOIN dbo.tipo_documento td ON (td.id_tipo_documento=c.id_tipo_documento) ");
            query.append("WHERE c.estado='1' and c.id_cliente=? ");
            query.append("ORDER BY c.denominacion ASC ");
		
            ps = conn.prepareStatement(query.toString());
            ps.setInt(1, idCliente);
            
            rs = ps.executeQuery();
            
            if(rs.next()){
            	obj=new Cliente();
            	obj.setIdCliente(rs.getInt("id_cliente"));
            	obj.setIdTipoCliente(rs.getInt("id_tipo_cliente"));
            	obj.setIdTipoDocumento(rs.getInt("id_tipo_documento"));
            	obj.setNroDocumento(rs.getString("nro_documento"));
            	obj.setDenominacion(rs.getString("denominacion"));
            	obj.setTelefono(rs.getString("telefono"));
            	obj.setCorreo(rs.getString("correo"));
            	obj.setDireccion(rs.getString("direccion"));
            	obj.setTipoCliente(rs.getString("tipo_cliente"));
            	obj.setTipoDocumento(rs.getString("tipo_documento"));
            }
            
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        
        return obj;
	}
	
	public List<Cliente> listaClientes(){
		List<Cliente> lista = new ArrayList<Cliente>(); 
		
		PreparedStatement ps=null;
		ResultSet rs=null;

        try {
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("SELECT c.id_cliente,c.id_tipo_cliente,c.id_tipo_documento,c.nro_documento,c.denominacion,c.telefono,c.correo,c.direccion, ");
            query.append("tc.denominacion tipo_cliente,td.denominacion tipo_documento ");
            query.append("FROM dbo.cliente c ");
            query.append("INNER JOIN dbo.tipo_cliente tc ON (tc.id_tipo_cliente=c.id_tipo_cliente) ");
            query.append("INNER JOIN dbo.tipo_documento td ON (td.id_tipo_documento=c.id_tipo_documento) ");
            query.append("WHERE c.estado='"+Constante.ESTADO_ACTIVO+"' ");
            query.append("ORDER BY c.denominacion ASC ");
		
            ps = conn.prepareStatement(query.toString());
            
            rs = ps.executeQuery();
            
            while(rs.next()){
            	Cliente obj=new Cliente();
            	obj.setIdCliente(rs.getInt("id_cliente"));
            	obj.setIdTipoCliente(rs.getInt("id_tipo_cliente"));
            	obj.setIdTipoDocumento(rs.getInt("id_tipo_documento"));
            	obj.setNroDocumento(rs.getString("nro_documento"));
            	obj.setDenominacion(rs.getString("denominacion"));
            	obj.setTelefono(rs.getString("telefono"));
            	obj.setCorreo(rs.getString("correo"));
            	obj.setDireccion(rs.getString("direccion"));
            	obj.setTipoCliente(rs.getString("tipo_cliente"));
            	obj.setTipoDocumento(rs.getString("tipo_documento"));
            	lista.add(obj);
            }
            
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        
        return lista;
	}
	
	
	public Integer validaNroDocumento(String nroDocumento){
		Integer result=Constante.ID_INICIAL; 
		
		PreparedStatement ps=null;
		ResultSet rs=null;

        try {
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("SELECT count(1) cantidad FROM dbo.cliente WHERE rtrim(ltrim(nro_documento))=? and estado='1'");
            ps = conn.prepareStatement(query.toString());
            ps.setString(1, nroDocumento.trim());
            rs = ps.executeQuery();
            
            if(rs.next()){
            	result=rs.getInt("cantidad");
            }
            
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        return result;
	}
	
	public Integer validaNroDocumento(String nroDocumento,Integer idCliente){
		Integer result=Constante.ID_INICIAL; 
		
		PreparedStatement ps=null;
		ResultSet rs=null;

        try {
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("SELECT count(1) cantidad FROM dbo.cliente where nro_documento=? and id_cliente!=? and estado='1'");
            ps = conn.prepareStatement(query.toString());
            ps.setString(1, nroDocumento.trim());
            ps.setInt(2, idCliente);
            rs = ps.executeQuery();
            
            if(rs.next()){
            	result=rs.getInt("cantidad");
            }
            
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        return result;
	}
	
	
	public Integer registraCliente(Integer idTipoCliente,Integer idTipoDocumento,String denominacion,
			   String nrodocumento,String telefono,String direccion,String email,Usuario pUsuario){
		PreparedStatement ps=null;
		ResultSet rs=null;
		Integer pk=0;
        try {
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("INSERT INTO dbo.cliente(id_tipo_cliente,id_tipo_documento,nro_documento,denominacion ");
            query.append(",telefono,direccion,correo,estado ");
            query.append(",audit_usuarioid,audit_fechahora,audit_terminal,audit_estado) ");
            query.append("VALUES ");
            query.append("(?,?,?,? ");
            query.append(",?,?,?,? ");
            query.append(",?,getdate(),?,1) ");
            
            String ids[] = new String[] { "id_cliente" };
            
            ps = conn.prepareStatement(query.toString(),ids);
            ps.setInt(1, idTipoCliente);
            ps.setInt(2, idTipoDocumento);
            ps.setString(3, nrodocumento);
	        ps.setString(4, denominacion);
	            ps.setString(5, telefono);
	            ps.setString(6, direccion);
	            ps.setString(7, email);
	            ps.setInt(8, Constante.ESTADO_ACTIVO);
            ps.setInt(9, pUsuario.getUsuarioId());
            ps.setString(10, Constante.TERMINAL);
            //--
            if(ps.executeUpdate()>0){
            	rs = ps.getGeneratedKeys();
                if ( rs.next() ) {
                    pk = rs.getInt(1);
                }
            }
            
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        
        return pk;
	}
	
	
	public Integer actualizaCliente(Integer idCliente,Integer idTipoCliente,Integer idTipoDocumento,String denominacion,
			   String nrodocumento,String telefono,String direccion,String email,Usuario pUsuario){
		PreparedStatement ps=null;
		ResultSet rs=null;
		Integer res=0;
        try {
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("UPDATE dbo.cliente ");
            query.append("SET id_tipo_cliente = ? ");
            query.append(",id_tipo_documento = ? ");
            query.append(",nro_documento = ? ");
            query.append(",denominacion = ? ");
	            query.append(",telefono = ? ");
	            query.append(",direccion = ? ");
	            query.append(",correo = ? ");
            query.append(",audit_usuarioid = ? ");
            query.append(",audit_fechahora = getdate() ");
            query.append(",audit_terminal = ? ");
            query.append("WHERE id_cliente=? ");
            
            ps = conn.prepareStatement(query.toString());
            ps.setInt(1, idTipoCliente);
            ps.setInt(2, idTipoDocumento);
            ps.setString(3, nrodocumento);
	        ps.setString(4, denominacion);
	            ps.setString(5, telefono);
	            ps.setString(6, direccion);
	            ps.setString(7, email);
            ps.setInt(8, pUsuario.getUsuarioId());
            ps.setString(9, Constante.TERMINAL);
            ps.setInt(10, idCliente);
            //--
            res=ps.executeUpdate();
            
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        
        return res;
	}
	
	public Integer eliminarCliente(Integer idCliente,Usuario pUsuario){
		PreparedStatement ps=null;
		ResultSet rs=null;
		Integer res=0;
        try {
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("UPDATE dbo.cliente ");
            query.append("SET estado = ? ");
            query.append(",audit_usuarioid = ? ");
            query.append(",audit_fechahora = getdate() ");
            query.append(",audit_terminal = ? ");
            query.append("WHERE id_cliente=? ");
            
            ps = conn.prepareStatement(query.toString());
            ps.setInt(1, Constante.ESTADO_ELIMINADO);
            ps.setInt(2, pUsuario.getUsuarioId());
            ps.setString(3, Constante.TERMINAL);
            ps.setInt(4, idCliente);
            //--
            res=ps.executeUpdate();
            
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        
        return res;
	}
	
	
	/**
	 * Lista clientes para el buscador en la guia
	 * @return
	 */
	public List<Value> listaClientesGuia(){
		List<Value> lista = new ArrayList<Value>(); 
		
		PreparedStatement ps=null;
		ResultSet rs=null;

        try {
        	Connection conn = ConnectionFactory.getInstance().getConnection();
        	
        	StringBuilder query = new StringBuilder();
            query.append("SELECT c.id_cliente,c.nro_documento,c.denominacion ");
            query.append(",(tc.denominacion+' con '+tp.denominacion+' '+c.nro_documento+', '+'Direccin en '+isnull(c.direccion,'-')) descripcion ");
            query.append("FROM dbo.cliente c  ");
            query.append("inner join dbo.tipo_documento tp on (c.id_tipo_documento=tp.id_tipo_documento) ");
            query.append("inner join dbo.tipo_cliente tc on (c.id_tipo_cliente=tc.id_tipo_cliente) ");
            query.append("WHERE c.estado='"+Constante.ESTADO_ACTIVO+"' ");
            query.append("ORDER BY c.denominacion ASC ");
            
            ps = conn.prepareStatement(query.toString());
            
            rs = ps.executeQuery();
            
            while(rs.next()){
            	Value obj=new Value();
            	obj.setId(rs.getInt("id_cliente"));
            	obj.setCodigo(rs.getString("nro_documento"));
            	obj.setDenominacion(rs.getString("denominacion"));
            	//--
            	obj.setDescripcion(rs.getString("descripcion"));
            	lista.add(obj);
            }
            
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        
        return lista;
	}
	
	public Value getCliente(Integer idCliente){
		Value obj = null; 
		
		PreparedStatement ps=null;
		ResultSet rs=null;
		
        try {
        	Connection conn = ConnectionFactory.getInstance().getConnection();
        	
        	StringBuilder query = new StringBuilder();
            query.append("SELECT c.id_cliente,c.nro_documento,c.denominacion ");
            query.append(",(tc.denominacion+' con '+tp.denominacion+' '+c.nro_documento+', '+'Direccin en '+isnull(c.direccion,'-')) descripcion,tp.id_tipo_documento ");
            query.append("FROM dbo.cliente c  ");
            query.append("inner join dbo.tipo_documento tp on (c.id_tipo_documento=tp.id_tipo_documento) ");
            query.append("inner join dbo.tipo_cliente tc on (c.id_tipo_cliente=tc.id_tipo_cliente) ");
            query.append("WHERE c.estado='1' and c.id_cliente=? ");
            
            ps = conn.prepareStatement(query.toString());
            ps.setInt(1, idCliente);
            
            rs = ps.executeQuery();
            
            if(rs.next()){
            	obj=new Value();
            	obj.setId(rs.getInt("id_cliente"));
            	obj.setCodigo(rs.getString("nro_documento"));
            	obj.setDenominacion(rs.getString("denominacion"));
            	//--
            	obj.setDescripcion(rs.getString("descripcion"));
            	obj.setIdTipoDocumento(rs.getInt("id_tipo_documento"));
            }
            
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        
        return obj;
	}
	
	
	public List<Value> listaListaPrecio(){
		List<Value> lista = new ArrayList<Value>(); 
		
		PreparedStatement ps=null;
		ResultSet rs=null;

        try {
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("SELECT id_lista_precio,denominacion FROM dbo.lista_precio ORDER BY denominacion ");
		    ps = conn.prepareStatement(query.toString());
            
            rs = ps.executeQuery();
            
            while(rs.next()){
            	Value obj=new Value();
            	obj.setId(rs.getInt("id_lista_precio"));
            	obj.setDenominacion(rs.getString("denominacion"));
            	lista.add(obj);
            }
            
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        
        return lista;
	}
	
	public List<Value> listaListaCliente(){
		List<Value> lista = new ArrayList<Value>(); 
		
		PreparedStatement ps=null;
		ResultSet rs=null;

        try {
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("SELECT DISTINCT c.id_cliente,c.denominacion ");
            query.append("FROM dbo.guia g  ");
            query.append("inner join dbo.cliente c ON (c.id_cliente=g.id_cliente)  ");
            query.append("left join dbo.guia_comprobante gc ON (gc.id_guia=g.id_guia) "); 
            query.append("WHERE (ISNULL(gc.id_guia,0)=0) and g.estado=2 ");
            query.append("ORDER BY c.denominacion ASC ");
            
		    ps = conn.prepareStatement(query.toString());
            
            rs = ps.executeQuery();
            
            while(rs.next()){
            	Value obj=new Value();
            	obj.setId(rs.getInt("id_cliente"));
            	obj.setDenominacion(rs.getString("denominacion"));
            	lista.add(obj);
            }
            
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        
        return lista;
	}
	
	
	public List<Value> listaPrioridad(){
		List<Value> lista = new ArrayList<Value>(); 
		
		PreparedStatement ps=null;
		ResultSet rs=null;

        try {
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("SELECT id_prioridad,denominacion FROM dbo.prioridad WHERE estado='1' ORDER BY id_prioridad ASC ");
		    ps = conn.prepareStatement(query.toString());
            
            rs = ps.executeQuery();
            
            while(rs.next()){
            	Value obj=new Value();
            	obj.setId(rs.getInt("id_prioridad"));
            	obj.setDenominacion(rs.getString("denominacion"));
            	lista.add(obj);
            }
            
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        
        return lista;
	}
	

	public String getDate(){
		String fecha=""; 
		
		PreparedStatement ps=null;
		ResultSet rs=null;

        try {
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("SELECT convert(varchar(20),getdate(),103) hoy ");
            ps = conn.prepareStatement(query.toString());
            
            rs = ps.executeQuery();
            
            if(rs.next()){
            	fecha=rs.getString("hoy");
            }
            
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        
        return fecha;
	}
	
	public Integer getDay(){
		Integer dia=0; 
		
		PreparedStatement ps=null;
		ResultSet rs=null;

        try {
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("select DATEPART(day,getdate()) currday ");
            ps = conn.prepareStatement(query.toString());
            
            rs = ps.executeQuery();
            
            if(rs.next()){
            	dia=rs.getInt("currday");
            }
            
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        
        return dia;
	}
	
	public String getDateFolder(String fechaComp){
		String fecha=""; 
		
		PreparedStatement ps=null;
		ResultSet rs=null;

        try {
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("SELECT convert(varchar(20),convert(date,?,103),112) hoy_folder ");
            
            ps = conn.prepareStatement(query.toString());
            ps.setString(1, fechaComp);
            
            rs = ps.executeQuery();
            
            if(rs.next()){
            	fecha=rs.getString("hoy_folder");
            }
            
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        
        return fecha;
	}
	
	
	
	

	//proceso dbf concar
	public Integer registraDbfGeneracion(String fechaArchivo,Integer cantidad){
		PreparedStatement ps=null;
		ResultSet rs=null;
		Integer result=0;
     try {
     	Connection conn = ConnectionFactory.getInstance().getConnection();
         
         StringBuilder query = new StringBuilder();
         query.append("INSERT INTO dbo.genera_dbf(fecha_generacion,fecha_archivo,cantidad,id_usuario)VALUES(getdate(),?,?,1)");
         
         ps = conn.prepareStatement(query.toString());
         ps.setDate(1, Util.stringToDate(fechaArchivo));
         ps.setInt(2, cantidad);
         //--
         result=ps.executeUpdate();
         
     } catch (SQLException e) {
         throw new RuntimeException(e);
     } catch (Exception e) {
         throw new RuntimeException(e);
     }finally {
     	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
         if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
     }
     
     return result;
	}
	
	public List<DbfGeneracion> listaDbfGeneracion(){
		List<DbfGeneracion> lista = new ArrayList<DbfGeneracion>(); 
		
		PreparedStatement ps=null;
		ResultSet rs=null;

        try {
        	Connection conn = ConnectionFactory.getInstance().getConnection();
            
            StringBuilder query = new StringBuilder();
            query.append("SELECT top 30 g.id_genera_dbf,dbo.getDateTimeAMPM(g.fecha_generacion) fecha_generacion,dbo.getDateDDMMYYYY(g.fecha_archivo) fecha_archivo,g.cantidad,u.username "); 
            query.append("FROM dbo.genera_dbf g ");
            query.append("INNER JOIN dbo.usuario u ON (g.id_usuario=u.id_usuario) ");
            query.append("ORDER BY g.fecha_generacion DESC ");
		    ps = conn.prepareStatement(query.toString());
            rs = ps.executeQuery();
            
            while(rs.next()){
            	DbfGeneracion obj=new DbfGeneracion();
            	obj.setIdGeneraDbf(rs.getInt("id_genera_dbf"));
            	obj.setFechaGeneracion(rs.getString("fecha_generacion"));
            	obj.setFechaArchivo(rs.getString("fecha_archivo"));
            	obj.setCantidad(rs.getInt("cantidad"));
            	obj.setUsuario(rs.getString("username"));
            	lista.add(obj);
            }
            
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } finally {
        	if (ps != null) {try {ps.close();} catch (SQLException e) {e.printStackTrace();}}
            if (rs != null) {try {rs.close();} catch (SQLException e) {e.printStackTrace();}}
        }
        
        return lista;
	}
	
}
